//PROFILE-NO
unit Misc;

// *****************************************************************************
// * Copyright 2003 mxbee                                                      *
// *****************************************************************************
// * This program is free software; you can redistribute it and/or modify      *
// * it under the terms of the GNU General Public License as published by      *
// * the Free Software Foundation; either version 2 of the License, or         *
// * (at your option) any later version.                                       *
// *                                                                           *
// * This program is distributed in the hope that it will be useful,           *
// * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
// * GNU General Public License for more details.                              *
// *                                                                           *
// * You should have received a copy of the GNU General Public License         *
// * along with this program; if not, write to the Free Software               *
// * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
// *****************************************************************************

{$INCLUDE CompilerOpts.pas}

interface

uses Windows,Forms,Dialogs,IniFiles,Classes;

{-DEFINE DEBUG_SSMIF_DISABLE_QUICKREAD}
{-DEFINE DEBUG_SSMIF_DISABLE_QUICKWRITE}

type
  TSafeSaveMemIniFile = class (TMemIniFile)
    FQuickWrite:        Boolean;
    FQuickWriteSection: String;
    FQuickWriteStrings: TStrings;
    FQuickRead:         Boolean;
    FQuickReadSection:  String;
    FQuickReadStrings:  TStrings;
    FQuickReadIndex:    Integer;
  public
    destructor Destroy; override;
    procedure UpdateFile; override;
    procedure WriteString(const Section, Ident, Value: String); override;
    function  ReadString(const Section, Ident, Default: String): String; override;
    procedure QuickWriteBegin(const NewSection: String = '');
    procedure QuickWriteNewSection(const NewSection: String);
    procedure QuickWriteEnd;
    procedure QuickReadBegin(const NewSection: String = '');
    procedure QuickReadNewSection(const NewSection: String);
    procedure QuickReadEnd;
  end;

procedure AdjustFormSize(Form: TForm);
procedure ShowDebugLines(Lines: String; Caption: String = 'Debug');
function  CommandLineOption(const OptionName: String): Boolean;
function  ExecuteSaveDialogSafely(Dlg: TSaveDialog): Boolean;

implementation

uses Controls,ComCtrls,StdCtrls,SysUtils;

// adjust these when recompiling with different desktop settings
// DeltaH = Form.Height - Form.ClientHeight
// DeltaW = Form.Width  - Form.ClientWidth   in design time mode.

const
  DESIGN_DELTAH = 27;
  DESIGN_DELTAW =  8;

type
  TAnchorInfo = record
    Control: TControl;
    Anchors: TAnchors;
    OrigXY:  TPoint;
    OrigWH:  TPoint;
  end;
  PAnchorInfo = ^TAnchorInfo;

procedure AdjustFormSize(Form: TForm);
var w,h: Integer;
begin
  w := Form.Width  - DESIGN_DELTAW;
  h := Form.Height - DESIGN_DELTAH;
  if Form.ClientWidth  <> w then Form.ClientWidth  := w;
  if Form.ClientHeight <> h then Form.ClientHeight := h;
end;

procedure ShowDebugLines(Lines,Caption: String);
var
  form:   TForm;
  riched: TRichEdit;
begin
  form := nil;
  try
    form := TForm.Create(Application);
    form.Position := poScreenCenter;
    form.Caption  := Caption;
    form.Width    := 600;
    form.Height   := 400;
    riched := TRichEdit.Create(form);
    riched.Parent := form;
    riched.Font.Name := 'Courier New';
    riched.Font.Size := 8;
    riched.Align := alClient;
    riched.ScrollBars := ssBoth;
    riched.WordWrap := False;
    riched.Text := Lines;
    form.ShowModal;
  finally
    form.Free;
  end;
end;

function  CommandLineOption(const OptionName: String): Boolean;
var i: Integer;
begin
  for i := 1 to ParamCount do
    if CompareText(ParamStr(i), OptionName) = 0 then begin
      Result := True; exit;
    end;
  Result := False;
end;

function  ExecuteSaveDialogSafely(Dlg: TSaveDialog): Boolean;
var t,t0: Int64;
begin
  // dirty workaround if the filename is invalid and save dialog doesn't show
  t0 := GetTickCount; Result := Dlg.Execute;
  t := GetTickCount; if t < t0 then t := t + $100000000;
  if (not Result) and ((t-t0) <= 300) then begin
    Dlg.FileName := '';
    Result := Dlg.Execute;
  end;
end;

{ TSafeSaveMemIniFile }

destructor TSafeSaveMemIniFile.Destroy;
begin
  if FQuickWrite then QuickWriteEnd; // frees list
  if FQuickRead  then QuickReadEnd;  // frees list
  inherited;
end;

procedure TSafeSaveMemIniFile.UpdateFile;
var
  OldName,TmpName: String;
begin
  // save to a .tmp file first
  OldName := FileName; TmpName := Filename + '.tmp';
  Rename(TmpName, False); // add .tmp to filename
  try
    inherited UpdateFile; // save data
    DeleteFile(OldName);
    if not RenameFile(TmpName, OldName) then raise Exception.Create('Rename failed for saving ini file');
  finally
    Rename(OldName, False); // rename back to old filename
  end;
end;

procedure TSafeSaveMemIniFile.QuickWriteBegin(const NewSection: String);
begin
  {$IFDEF DEBUG_SSMIF_DISABLE_QUICKWRITE} exit; {$ENDIF}
  Assert(not FQuickWrite);
  FQuickWriteStrings := TStringList.Create;
  GetStrings(FQuickWriteStrings);
  FQuickWrite := True;
  FQuickWriteSection := '';
  if NewSection <> '' then QuickWriteNewSection(NewSection);
end;

procedure TSafeSaveMemIniFile.QuickWriteEnd;
begin
  {$IFDEF DEBUG_SSMIF_DISABLE_QUICKWRITE} exit; {$ENDIF}
  Assert(FQuickWrite);
  SetStrings(FQuickWriteStrings);
  FQuickWriteStrings.Free;
  FQuickWrite := False;
end;

procedure TSafeSaveMemIniFile.QuickWriteNewSection(const NewSection: String);
begin
  {$IFDEF DEBUG_SSMIF_DISABLE_QUICKWRITE} exit; {$ENDIF}
  Assert(FQuickWrite);
  FQuickWriteStrings.Add('[' + NewSection + ']');
  FQuickWriteSection := NewSection;
end;

procedure TSafeSaveMemIniFile.WriteString(const Section, Ident, Value: String);
begin
  if not FQuickWrite then
    inherited
  else begin
    Assert(Section = FQuickWriteSection);
    FQuickWriteStrings.Add(Ident + '=' + Value);
  end;
end;

procedure TSafeSaveMemIniFile.QuickReadBegin(const NewSection: String);
begin
  {$IFDEF DEBUG_SSMIF_DISABLE_QUICKREAD} exit; {$ENDIF}
  Assert(not FQuickRead);
  FQuickReadStrings := TStringList.Create;
  FQuickRead := True;
  FQuickReadSection := '';
  FQuickReadIndex   := 0;
  if NewSection <> '' then QuickReadNewSection(NewSection);
end;

procedure TSafeSaveMemIniFile.QuickReadEnd;
begin
  {$IFDEF DEBUG_SSMIF_DISABLE_QUICKREAD} exit; {$ENDIF}
  Assert(FQuickRead);
  FQuickReadStrings.Free;
  FQuickRead := False;
end;

procedure TSafeSaveMemIniFile.QuickReadNewSection(const NewSection: String);
begin
  {$IFDEF DEBUG_SSMIF_DISABLE_QUICKREAD} exit; {$ENDIF}
  Assert(FQuickRead);
  ReadSectionValues(NewSection, FQuickReadStrings);
  FQuickReadSection := NewSection;
  FQuickReadIndex   := 0;
end;

function TSafeSaveMemIniFile.ReadString(const Section, Ident, Default: String): String;
var
  ok: Boolean;
begin
  if not FQuickRead then
    Result := inherited ReadString(Section,Ident,Default)
  else begin
    Assert(Section = FQuickReadSection);
    ok := (FQuickReadIndex >= 0) and (FQuickReadIndex < FQuickReadStrings.Count);
    if ok then ok := AnsiCompareText(FQuickReadStrings.Names[FQuickReadIndex], Ident) = 0;
    if ok then begin
      Result := Copy(FQuickReadStrings.Strings[FQuickReadIndex], Length(Ident)+2, MaxInt);
      inc(FQuickReadIndex);
    end else
      Result := inherited ReadString(Section,Ident,Default)
  end;
end;

end.
